#ifndef _AFFIXMGR_HXX_
#define _AFFIXMGR_HXX_
#include <cstdlib>
#include <cstring>
#include <cstdio>

#include "atypes.hxx"
#include "baseaffix.hxx"
#include "hashmgr.hxx"
#include "config.hxx"

class AffixMgr
{

  AffEntry *          pStart[SETSIZE];
  AffEntry *          sStart[SETSIZE];
  HashMgr *           pHMgr;
  char *              trystring;
  char *              encoding;
  char                compound;
  char                compound2;
  char                compound3;
  char *              compoundword;
  char                compoundforbidflag;
  char                forbiddenword;
  char                onlyroot;
  int                 cpdmin;
  int                 numrep;
  replentry *         reptable;
  int                 cpdwordmax;
  int                 cpdmaxsyllable;
  char *              cpdvowels;
  int                 cpdvowelslen;
  replentry *         accent;
  char *              cpdsyllablenum;
  const char *        pfxappnd; // BUG: not stateless
  const char *        sfxappnd; // BUG: not stateless
  char	              sfxflag;  // BUG: not stateless
  AffEntry *          sfx;      // BUG: not stateless
  AffEntry *          pfx;      // BUG: not stateless
  int                 checknum;
  char *              wordchars;
  char                hu_kotohangzo;
  char *              version;

  char	        sfxword[MAXWORDLEN+1]; // global for root generation (HU) XXX (not stateless)
  
  int                 havecontclass; // boolean variable
  short               contclasses[SETSIZE]; // flags of possible continuing classes (double affix)
    
public:
 
  AffixMgr(const char * affpath, HashMgr * ptr);
  ~AffixMgr();
  struct hentry *     affix_check(const char * word, int len, const char needflag = '\0');
  struct hentry *     prefix_check(const char * word, int len, int prefix_in_cpms, const char needflag = '\0');
  struct hentry *     prefix_check_twosfx(const char * word, int len, int prefix_in_cpms);
  struct hentry *     suffix_check(const char * word, int len, int sfxopts, AffEntry* ppfx,
			char ** wlst, int maxSug, int * ns, const char cclass = '\0',
                        const char needflag = '\0', int prexix_in_cpms = 1);
  struct hentry *     suffix_check_twosfx(const char * word, int len, int sfxopts, AffEntry* ppfx);

  char * affix_check_morph(const char * word, int len, const char needflag = '\0');
  char * prefix_check_morph(const char * word, int len, int prefix_in_cpms);
  char * suffix_check_morph (const char * word, int len, int sfxopts, AffEntry * ppfx,
            const char cclass='\0', const char needflag = '\0');

  char * prefix_check_twosfx_morph(const char * word, int len, int prefix_in_cpms);
  char * suffix_check_twosfx_morph(const char * word, int len, int sfxopts, AffEntry * ppfx);


  int                 get_syllable (const char * word, int wlen);
  int                 repl_check(const char * word, int len);
//  int                 swap_check(const char * word, int len);
  int                 candidate_check(const char * word, int len);
  struct hentry *     compound_check(const char * word, int len, char compound_flag, 
                              int wordnum, int numsyllable, int maxwordnum, int hu_mov_rule,
			      int * cmpdstemnum, int * cmpdstem);

  int compound_check_morph(const char * word, int len, char compound_flag, 
                              int wordnum, int numsyllable, int maxwordnum, int hu_mov_rule,
                              char ** result, char * partresult);

  struct hentry *     lookup(const char * word);
  int                 get_numrep();
  struct replentry *  get_reptable();
  struct replentry *  get_accent();
  char *              get_encoding();
  char *              get_try_string();
  char *              get_wordchars();
  char                get_compound();
  char                get_forbiddenword();
  char                get_onlyroot();
  char *              get_compoundword();
  int                 get_checknum();
  char *              get_possible_root();
  const char *        get_prefix();
  const char *        get_suffix();
  const char *        get_version();
  const int           have_contclass();

private:
  int  parse_file(const char * affpath);
  int  parse_try(char * line);
  int  parse_set(char * line);
  int  parse_cpdflag(char * line);
  int  parse_cpdfirst(char * line);
  int  parse_cpdlast(char * line);
  int  parse_cpdword(char * line);
  int  parse_cpdforbid(char * line);
  int  parse_forbid(char * line);
  int  parse_onlyroot(char * line);
  int  parse_cpdmin(char * line);
  int  parse_cpdsyllable(char * line);
  int  parse_accent(char * line);
  int  parse_syllablenum(char * line);
  int  parse_reptable(char * line, FILE * af);
  int  parse_affix(char * line, const char at, FILE * af);
  int  parse_wordchars(char * line);
  int  parse_hu_kotohangzo(char * line);
  int  parse_version(char * line);

  void encodeit(struct affentry * ptr, char * cs);
  int build_pfxlist(AffEntry* pfxptr);
  int build_sfxtree(AffEntry* sfxptr);
  int process_pfx_order();
  int process_sfx_order();
  AffEntry * process_sfx_inorder(AffEntry * ptr, AffEntry * ptr);
  int process_sfx_tree_to_list();
  
};

#endif

